@@ -1,11 +1,11 @@
-__version__ = "0.2.2"
+__version__ = "0.2.4"
 
 from .anansescanpy import export_CPM_scANANSE
 from .anansescanpy import export_ATAC_scANANSE
 from .anansescanpy import config_scANANSE
 from .anansescanpy import DEGS_scANANSE
 from .anansescanpy import per_cluster_df
-from .anansescanpy import export_ATAC_maelstrom
 from .anansescanpy import import_scanpy_scANANSE
 from .anansescanpy import Maelstrom_Motif2TF
-from .anansescanpy import import_scanpy_maelstrom
\ No newline at end of file
+from .anansescanpy import import_scanpy_maelstrom
+from .anansescanpy import Factor_Motif_Plot
\ No newline at end of file
